﻿using System;
using System.Collections.Generic;
using IMSES_Template.DataAccess;
using IMSES_Template.Model;

namespace IMSES_Sample
{
    class Example
    {
        // Please fill in the values below, as theses are needed to run the examples:
        private const string UserName = "8ce40262-24d1-4e6c-b13a-19c213d514fa";
        private const string Password = "xvhcimDfCfDZzQU";
        //private const ItslearningServers Server = ItslearningServers.Test;
        // Or direct it to your local ImsEs file, i.e. 
        private const string Server = "https://localhost:446/WCFServiceLibrary/ImsEnterpriseServicesPort.svc";

        static void Main()
        {
            // In this file two examples are shown, to familiarize yourself with the functions in this template.

            // Before the examples below can be executed however, you should set up ImsEs on the Itslearning sever 
            // you are going to use, and fill in the username and password in the fields above. Instructions on how 
            // to set-up ImsEs in Itslearning can be found in the official itslearning IMS-ES documentation. There
            // you can also find more documentation on the exact functions in this template.

            // Note: The examples below can only be run once, the second time the results will be different, as some
            // of the objects will still be in the trash can of Itslearning.

            try
            {
                // A simple example, to show the basic functionality of the ImsEs Template.
                //SimpleExample();

                // A bit more advanced example, to show the more difficult functionality.
                // (please fill in the SyncId of the main organisation if the site you are testing)
                //AdvancedExample("");

                //ReadAllUsersExample();

                // (please fill in the SyncId of the main organisation if the site you are testing)
                SetOrganisationLegalIdExample("root");
            }
            catch (Exception ex)
            {
                if (ex is ImsEsBatchException batchEx)
                {
                    foreach (OperationResult errorResponse in batchEx.BatchOperationResult.ErrorMessages)
                    {
                        Console.WriteLine("{0} - {1}", errorResponse.Identifier, errorResponse.Message);
                    }
                }
                else
                {
                    Console.WriteLine(ex.ToString());
                }
            }

            // Make sure the console doesn't disappear immediately.
            Console.ReadKey();
        }

        /// <summary>
        /// A simple example, showing the basics in using the ImsEs Template to communicate with Itslearning.
        /// </summary>
        private static void SimpleExample()
        {
            Console.WriteLine("-start: Simple Example-");

            Console.WriteLine("\n" + "Start with setting up an ImsEs Client.");
            using (var client = new ImsEsClient(UserName, Password, Server) { MessageIdentifier = "SimpleExample" })
            {
                Console.WriteLine("\n" + "Let's start by creating a new User object.");

                // Create a new user
                var bigBird = new User("big_bird")
                {
                    FirstName = "Big",          // Required
                    LastName = "Bird",          // Required
                    UserName = "b.bird",        // Required
                    Profile = Profile.Guest,    // Required, but initialized by default to Guest.

                    Address = new Address
                    {
                        AddressLine1 = "123a Sesame Street",
                        AddressLine2 = "(large nest behind 123 Sesame Street)",
                        PostalCode = "10128",
                        City = "New York"
                    }
                };
                Console.WriteLine(" - Big Bird created!");
                Console.WriteLine("\n" + "Now to add Big Bird to Itslearning.");
                OperationResult warning = client.CreateUser(bigBird);
                Console.WriteLine(warning.Status == ResultStatus.Warning ? warning.Message : " - Big Bird added successfully!");

                // Update user info
                Console.WriteLine("\n" + "But we forgot to add an email-address to Big Bird.");
                bigBird.Email = "big.bird@sesamestreet.com";
                Console.WriteLine(" - Email address added.");
                Console.WriteLine("\n" + "Also make sure that the email is added in Itslearning.");
                warning = client.UpdateUser(bigBird);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Big Bird updated successfully!");

                // Read user info
                Console.WriteLine("\n" + "Now we can also check if the update arrived in Itslearning correctly.");
                warning = client.ReadUser("big_bird", out User testUser);
                if (!string.IsNullOrEmpty(warning.Message))
                {
                    Console.WriteLine(warning.Message);
                }

                Console.WriteLine("\n" + "Big Bird has the following email address in Itslearning:");
                Console.WriteLine(" - " + testUser.Email);

                // Read hierarchies for the user
                Console.WriteLine("\n" + "Let's check to which Hierarchies Big Bird has been added.");
                warning = client.ReadHierarchiesFromUser("big_bird", out List<Hierarchy> hierarchyList);
                if (!string.IsNullOrEmpty(warning.Message))
                {
                    Console.WriteLine(warning.Message);
                }

                Console.WriteLine("\n" + "Big Bird is enrolled in the following Hierarchies:");
                foreach (Hierarchy hierarchy in hierarchyList)
                {
                    Console.WriteLine(" - " + hierarchy.Title);
                }

                Console.WriteLine("\n" + "It was just the main organisation, as expected.");
                Hierarchy main = hierarchyList[0];

                // Create a new hierarchy
                Console.WriteLine("\n" + "So create a new Hierarchy, so we can add Big Bird to it.");
                var newHierarchy = new Hierarchy("sesameCreatures")
                {
                    Title = "Sesame street Creatures",       // Required.
                    ParentHierarchyId = main.SyncId          // Required, an Hierarchy always needs a parent.
                };

                warning = client.CreateHierarchy(newHierarchy);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - New Hierarchy added successfully!");

                // Create membership for the user in the created hierarchy
                Console.WriteLine("\n" + "Now we can add Big Bird to the hierarchy.");
                warning = client.CreateMembership("big_bird", "sesameCreatures", HierarchyRole.Guest);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Big Bird added successfully!");

                Console.WriteLine("\n" + "We can check the Hierarchies of Big Bird again.");

                // Read hierarchies for the user after adding it to the newly created
                warning = client.ReadHierarchiesFromUser("big_bird", out hierarchyList);
                if (!string.IsNullOrEmpty(warning.Message))
                {
                    Console.WriteLine(warning.Message);
                }

                Console.WriteLine("\n" + "Big Bird is enrolled in the following Hierarchies:");
                foreach (Hierarchy hierarchy in hierarchyList)
                {
                    Console.WriteLine(" - " + hierarchy.Title);
                }

                // Delete hierarchy
                Console.WriteLine("\n" + "Sadly no animals were allowed at the school, so the Hierarchy was disbanded.");
                warning = client.DeleteHierarchy("sesameCreatures");
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Hierarchy deleted.");

                // Delete user
                Console.WriteLine("\n" + "And also Big Bird leaves the school.");
                warning = client.DeleteUser("big_bird");
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Big Bird deleted.");

                Console.WriteLine("\n" + "(in case you need it for the advanced example, the main organisation is \"" + main.SyncId + "\")");
            }

            Console.WriteLine("\n" + "-end-");
        }

        private static void ReadAllUsersExample()
        {
            Console.WriteLine("-start: Read all users Example-");
            using (var client = new ImsEsClient(UserName, Password, Server) {MessageIdentifier = "ReadAllUsersExample" })
            {
                Console.Write("\n" + "With ReadAllUsers method you can read all users info using paging, like shown in the simple example. ");
                Console.Write("\n" + "The index for the first page is 1. The page size can be equal or less than 100.");

                var pageIndex = 1;
                const int pageSize = 100;

                Console.Write("\n" + "Let's try to read the first page");
                OperationResult warning = client.ReadAllUsers(pageIndex, pageSize, out List<User> users, out int totalUsers);
                if (!string.IsNullOrEmpty(warning.Message))
                {
                    Console.WriteLine(warning.Message);
                }
                else
                {
                    Console.Write("\n" + $"We read {users.Count} users. Total count of users is {totalUsers}.");
                }

                // Read all users in a loop
                var allUsers = new List<User>();
                while (allUsers.Count < totalUsers)
                {
                    warning = client.ReadAllUsers(pageIndex, pageSize, out users, out totalUsers);
                    if (!string.IsNullOrEmpty(warning.Message))
                    {
                        Console.WriteLine(warning.Message);
                    }
                    else
                    {
                        allUsers.AddRange(users);
                        Console.Write("\n" + $"We read page {pageIndex} ({allUsers.Count} users from {totalUsers}).");
                        if (users.Count > 0)
                        {
                            Console.Write("\n" + $"Let's see the first and last name for the first user in the page: {users[0].FirstName} {users[0].LastName}");
                        }
                    }
                    pageIndex++;
                }
                Console.WriteLine("\n" + $"We read all {allUsers.Count} users from {totalUsers}.");
            }

            Console.WriteLine("\n" + "-end-");
        }

        private static void SetOrganisationLegalIdExample(string mainOrganisation)
        {
            Console.WriteLine("-start: Set LegalID for organisations example-");
            Console.Write("\n" + "With ImsEs you can create Hierarchies, like shown in the example. ");

            using (var client = new ImsEsClient(UserName, Password, Server) {MessageIdentifier = "SetOrganisationLegalId" })
            {
                // Create a new hierarchy
                Console.Write("\n Create a new organization (school): 'School A' with SyncId = 'SchoolA_SyncId' and LegalId='SchoolA_LegalId'. ");
                var newSchool = new Hierarchy("SchoolA_SyncId", true)
                {
                    Title = "School A",
                    ParentHierarchyId = mainOrganisation,
                    LegalId = "SchoolA_LegalId"
                };

                OperationResult createResult = client.CreateHierarchy(newSchool);
                Console.WriteLine(!string.IsNullOrEmpty(createResult.Message) ? createResult.Message : "\n - New school created successfully.");

                Console.Write("\n Let's read a new created organization (school): ");
                client.ReadHierarchy("SchoolA_SyncId", out Hierarchy readHierarchy);
                if (readHierarchy != null)
                {
                    Console.Write($"\n Title: {readHierarchy.Title}");
                    Console.Write($"\n Type: {readHierarchy.GroupType}");
                    Console.Write($"\n SyncId: {readHierarchy.SyncId}");
                    Console.Write($"\n LegalId: {readHierarchy.LegalId}");
                }

                // Updates a legal id for an organization
                Console.Write("\n" + "Updates created organization (school): 'School A' with a new LegalId='SchoolA_LegalId2'. ");
                newSchool.LegalId = "SchoolA_LegalId2";
                OperationResult updateResult = client.UpdateHierarchy(newSchool);
                Console.WriteLine(!string.IsNullOrEmpty(updateResult.Message) ? updateResult.Message : "\n - School is updated successfully.");
                Console.Write("\n" + "Let's read the updated organization (school): ");
                client.ReadHierarchy("SchoolA_SyncId", out readHierarchy);
                Console.Write($"\n LegalId: {readHierarchy.LegalId}");

                // Delete organization
                client.DeleteHierarchy("SchoolA_SyncId");
                Console.WriteLine("\n" + "-end-");
            }
        }

        /// <summary>
            /// A bit more advanced example, showing somewhat more difficult functions from the ImsEs Template.
            /// </summary>
            /// <param name="mainOrganisation"></param>
            private static void AdvancedExample(string mainOrganisation)
        {
            Console.WriteLine("-start: Advanced Example-");

            using (var client = new ImsEsClient(UserName, Password, Server) { MessageIdentifier = "AdvancedExample" })
            {
                Console.Write("\n" + "With ImsEs you can create Hierarchies, like shown in the simple example. ");
                Console.WriteLine("But you can also create new organisations, which works in almost the same way.");

                // Create a new hierarchy
                var newOrganisation = new Hierarchy("occult666", true)
                {
                    Title = "Occult Department",
                    ParentHierarchyId = mainOrganisation,
                };

                OperationResult warning = client.CreateHierarchy(newOrganisation);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - New organisation created successfully.");
                Console.WriteLine("\n" + "In this new organisation we will create a new course, Magic 101.");

                // Create a course
                var newHierarchy = new Hierarchy("m101")
                {
                    Title = "Magic 101 group",
                    ParentHierarchyId = "occult666"
                };

                var newCourse = new Course("m101")
                {
                    Title = "Magic 101",
                    Code = "M101",
                    LinkedHierarchy = newHierarchy
                };

                warning = client.CreateCourse(newCourse);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - New course created successfully.");

                Console.Write("\n" + "As you might have seen from the object definition, a course is actually linked to ");
                Console.Write("an hierarchy. When you create a Course, you create a Hierarchy, with some extra information, ");
                Console.WriteLine("which Itslearning then converts to a new course (plus the new hierarchy of course).");

                Console.WriteLine("\n" + "Drawn to the school by this new Course, four new students apply.");

                // Create users
                var user1 = new User("pHarry")
                {
                    FirstName = "Harry",
                    LastName = "Potter",
                    UserName = "h.potter",
                    Profile = Profile.Pupil
                };
                Console.WriteLine(" - Harry Potter");

                var user2 = new User("wMerlin")
                {
                    FirstName = "Merlin",
                    LastName = "the Wizard",
                    UserName = "merlin",
                    Profile = Profile.Pupil
                };
                Console.WriteLine(" - Merlin");

                var user3 = new User("gGandalf")
                {
                    FirstName = "Gandalf",
                    LastName = "the Grey",
                    UserName = "g.grey",
                    Profile = Profile.Pupil
                };
                Console.WriteLine(" - Gandalf the Grey");

                var user4 = new User("hCatweazle")
                {
                    FirstName = "Catweazle",
                    LastName = "the Hermit",
                    UserName = "catweazle",
                    Profile = Profile.Pupil
                };
                Console.WriteLine(" - Catweazle");

                BatchOperationResult warnings = client.CreateUsers(new[] { user1, user2, user3, user4 });
                if (warnings.WarningMessages != null)
                {
                    foreach (OperationResult warningResult in warnings.WarningMessages)
                    {
                        Console.WriteLine(warningResult.Identifier + " - " + warningResult.Message);
                    }
                }

                Console.WriteLine("\n" + "Four Wizards created successfully.");

                Console.WriteLine("\n" + "The wizards immediately apply to the course.");
                // Add these users to the hierarchy
                var membershipList = new[]
                    {
                        new MembershipPair
                            {
                                UserSyncKey = "pHarry",
                                HierarchySyncId = "m101",
                                Role = HierarchyRole.Pupil
                            },
                        new MembershipPair
                            {
                                UserSyncKey = "wMerlin",
                                HierarchySyncId = "m101",
                                Role = HierarchyRole.Pupil
                            },
                        new MembershipPair
                            {
                                UserSyncKey = "gGandalf",
                                HierarchySyncId = "m101",
                                Role = HierarchyRole.Pupil
                            },
                        new MembershipPair
                            {
                                UserSyncKey = "hCatweazle",
                                HierarchySyncId = "m101",
                                Role = HierarchyRole.Pupil
                            }
                    };

                warnings = client.CreateMemberships(membershipList);
                if (warnings.WarningMessages != null)
                {
                    foreach (OperationResult warningResult in warnings.WarningMessages)
                    {
                        Console.WriteLine(warningResult.Identifier + " - " + warningResult.Message);
                    }
                }

                Console.WriteLine("\n" + "Four Wizards added to the course.");
                Console.WriteLine("\n" + "Check if they were added correctly:");

                // Read users from the hierarchy
                client.ReadUsersFromHierarchy("m101", out List<User> userList);
                Console.WriteLine("The following Users are enrolled in Magic 101:");
                foreach (User user in userList)
                {
                    Console.WriteLine(" - " + user.FirstName + " " + user.LastName);
                }

                Console.Write("\n" + "As you can see, also the adminstartor(s) of the main organisation are listed here,");
                Console.WriteLine(" they are automatically added to any hierarchy or Course that is added to the organisation they are administrator of.");

                Console.WriteLine("\n" + "Due to his old age, Gandalf was chosen to be teacher of the Course.");

                // Update membership - change role in the hierarchy for the user
                warning = client.UpdateMembership("gGandalf", "m101", HierarchyRole.Teacher);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Gandalf successful added as Teacher.");

                Console.WriteLine("\n" + "Harry Potter wants to be ingocnito, and therefore requests a different SyncKey.");

                // Change user sync key
                warning = client.ChangeUserKey("pHarry", "hp347");
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : " - Harry Potters UserKey changed.");

                Console.WriteLine("\n" + "Now to check if his old key has indeed been removed:");
                Console.WriteLine("(this will generate an exception, as expected.)");

                try
                {
                    client.ReadUser("pHarry", out User _);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("\n" + ex);
                }


                Console.WriteLine("\n" + "To illustrate the difference between an Update and a Replace, Merlin needs some more information added.");

                // Update the user
                user2.Phone = "1234566789";
                client.UpdateUser(user2);

                Console.WriteLine("\n" + "Before the Update:");
                Console.WriteLine(" - Email = " + user2.Email);
                Console.WriteLine(" - Phone = " + user2.Phone);

                user2.Phone = null;
                user2.Email = "merling@avalon.co.uk";
                client.UpdateUser(user2);
                client.ReadUser("wMerlin", out user2);

                Console.WriteLine("\n" + "After the Update:");
                Console.WriteLine(" - Email = " + user2.Email);
                Console.WriteLine(" - Phone = " + user2.Phone);

                user2.Email = "merlin@avalon.co.uk";
                client.ReplaceUser(user2);
                client.ReadUser("wMerlin", out user2);

                Console.WriteLine("\n" + "After the Replace:");
                Console.WriteLine(" - Email = " + user2.Email);
                Console.WriteLine(" - Phone = " + user2.Phone);

                Console.WriteLine("\n" + "Because the wizards do not want to have anything to with the school, except their course, they change their organisation.");

                membershipList = new[]
                    {
                        new MembershipPair { UserSyncKey = "hp347", HierarchySyncId = "occult666" },
                        new MembershipPair { UserSyncKey = "wMerlin", HierarchySyncId = "occult666" },
                        new MembershipPair { UserSyncKey = "gGandalf", HierarchySyncId = "occult666" },
                        new MembershipPair { UserSyncKey = "hCatweazle", HierarchySyncId = "occult666" }
                    };

                warnings = client.SetHomeOrganisations(membershipList);
                if (warnings.WarningMessages != null)
                {
                    foreach (OperationResult warningResult in warnings.WarningMessages)
                    {
                        Console.WriteLine(warningResult.Identifier + " - " + warningResult.Message);
                    }
                }

                Console.WriteLine("A few days before the start of the semester, the school finds out they made a mistake.");
                Console.WriteLine("It seems the course was actually called Arabic 101, but due to some bad handwriting it was entered incorrectly.");

                // Update the course info
                newCourse.Title = "Arabic 101";
                newCourse.Code = "A101";

                warning = client.UpdateCourse(newCourse);
                Console.WriteLine(!string.IsNullOrEmpty(warning.Message) ? warning.Message : "- Course renamed correctly.");

                Console.WriteLine("\n" + "When the four wizards hear about the change, they all leave the school.");

                // Delete users
                warnings = client.DeleteUsers(new[] { "hp347", "wMerlin", "gGandalf", "hCatweazle" });
                if (warnings.WarningMessages != null)
                {
                    foreach (OperationResult warningResult in warnings.WarningMessages)
                    {
                        Console.WriteLine(warningResult.Identifier + " - " + warningResult.Message);
                    }
                }

                Console.WriteLine(" - Four wizards deleted successfully.");

                Console.WriteLine("\n" + "With the students gone, the school decides to also delete the course.");
                Console.Write("But here it gets tricky. Through ImsEs you can only delete the linked Hierarchy, ");
                Console.Write("never the course itself. So this will have to be done manually, through the Itslearning UI.");
                Console.WriteLine();
                Console.Write("If you do delete the hierarchy, the Course will remain in Itslearning, but then you will have ");
                Console.WriteLine("no way to change it through ImsEs anymore.");

                // Delete hierarchy
                client.DeleteHierarchy("m101");
            }

            Console.WriteLine("\n" + "-end-");
        }

    }
}
