﻿using System;
using System.Runtime.Serialization;
using System.Security.Permissions;
using IMSES_Template.Model;

namespace IMSES_Template.DataAccess
{
    [Serializable]
    public class ImsEsException : Exception
    {
        public OperationResult OperationResult { get; set; }

        public ImsEsException(string message)
            : base(message)
        {
        }

        public ImsEsException(OperationResult result)
            : base(result.Message)
        {
            OperationResult = result;
        }

        // Serialization:
        protected ImsEsException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            if (info != null)
            {
                OperationResult = (OperationResult) info.GetValue("OperationResult", typeof (OperationResult));
            }
        }

        [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);

            // And add the OperationResult
            info.AddValue("OperationResult", OperationResult);
        }

    }

    [Serializable]
    public class ImsEsBatchException : Exception
    {
        public BatchOperationResult BatchOperationResult { get; set; }

        public ImsEsBatchException(string message)
            : base(message)
        {
        }

        public ImsEsBatchException(string message, BatchOperationResult result)
            : base(message)
        {
            BatchOperationResult = result;
        }

        // Serialization:
        protected ImsEsBatchException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            if (info != null)
            {
                BatchOperationResult = (BatchOperationResult) info.GetValue("BatchOperationResult", typeof (BatchOperationResult));
            }
        }

        [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);

            // And add the BatchOperationResult
            info.AddValue("BatchOperationResult", BatchOperationResult);
        }
    }
}
 